@@ -28,6 +28,11 @@
 #include <cmath>
 #include <vector>
 
+#include "config-OpenMP.h"
+#if HAVE_OpenMP
+#include <omp.h>
+#endif
+
 #define return_fallback(value)                                                                                                                                 \
     if (m_imageData.m_samples.size() == 0) {                                                                                                                   \
         return value;                                                                                                                                          \
@@ -55,6 +60,12 @@ ImageColors::~ImageColors()
 
 void ImageColors::setSource(const QVariant &source)
 {
+    if (m_futureSourceImageData) {
+        m_futureSourceImageData->cancel();
+        m_futureSourceImageData->deleteLater();
+        m_futureSourceImageData = nullptr;
+    }
+
     if (source.canConvert<QQuickItem *>()) {
         setSourceItem(source.value<QQuickItem *>());
     } else if (source.canConvert<QImage>()) {
@@ -62,7 +73,29 @@ void ImageColors::setSource(const QVariant &source)
     } else if (source.canConvert<QIcon>()) {
         setSourceImage(source.value<QIcon>().pixmap(128, 128).toImage());
     } else if (source.canConvert<QString>()) {
-        setSourceImage(QIcon::fromTheme(source.toString()).pixmap(128, 128).toImage());
+        const QString sourceString = source.toString();
+
+        if (QIcon::hasThemeIcon(sourceString)) {
+            setSourceImage(QIcon::fromTheme(sourceString).pixmap(128, 128).toImage());
+        } else {
+            QFuture<QImage> future = QtConcurrent::run([sourceString]() {
+                if (auto url = QUrl(sourceString); url.isLocalFile()) {
+                    return QImage(url.toLocalFile());
+                }
+                return QImage(sourceString);
+            });
+            m_futureSourceImageData = new QFutureWatcher<QImage>(this);
+            connect(m_futureSourceImageData, &QFutureWatcher<QImage>::finished, this, [this, source]() {
+                const QImage image = m_futureSourceImageData->future().result();
+                m_futureSourceImageData->deleteLater();
+                m_futureSourceImageData = nullptr;
+                setSourceImage(image);
+                m_source = source;
+                Q_EMIT sourceChanged();
+            });
+            m_futureSourceImageData->setFuture(future);
+            return;
+        }
     } else {
         return;
     }
@@ -164,6 +197,9 @@ void ImageColors::update()
     if (!m_sourceItem) {
         if (!m_sourceImage.isNull()) {
             runUpdate();
+        } else {
+            m_imageData = {};
+            Q_EMIT paletteChanged();
         }
         return;
     }
@@ -225,39 +261,59 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
     imageData.m_clusters.clear();
     imageData.m_samples.clear();
 
-    QColor sampleColor;
+#if HAVE_OpenMP
+    static const int numCore = std::min(8, omp_get_num_procs());
+    omp_set_num_threads(numCore);
+    std::vector<decltype(imageData.m_samples)> tempSamples(numCore, decltype(imageData.m_samples){});
+#endif
     int r = 0;
     int g = 0;
     int b = 0;
     int c = 0;
+
+#pragma omp parallel for collapse(2) reduction(+ : r) reduction(+ : g) reduction(+ : b) reduction(+ : c)
     for (int x = 0; x < sourceImage.width(); ++x) {
         for (int y = 0; y < sourceImage.height(); ++y) {
-            sampleColor = sourceImage.pixelColor(x, y);
+            const QColor sampleColor = sourceImage.pixelColor(x, y);
             if (sampleColor.alpha() == 0) {
                 continue;
             }
             if (ColorUtils::chroma(sampleColor) < 20) {
                 continue;
             }
-
             QRgb rgb = sampleColor.rgb();
-            c++;
+            ++c;
             r += qRed(rgb);
             g += qGreen(rgb);
             b += qBlue(rgb);
+#if HAVE_OpenMP
+            tempSamples[omp_get_thread_num()] << rgb;
+#else
             imageData.m_samples << rgb;
-            positionColor(rgb, imageData.m_clusters);
+#endif
         }
+    } // END omp parallel for
+
+#if HAVE_OpenMP
+    for (auto &s : tempSamples) {
+        imageData.m_samples << std::move(s);
     }
+#endif
 
     if (imageData.m_samples.isEmpty()) {
         return imageData;
     }
 
+    for (QRgb rgb : std::as_const(imageData.m_samples)) {
+        positionColor(rgb, imageData.m_clusters);
+    }
+
     imageData.m_average = QColor(r / c, g / c, b / c, 255);
 
     for (int iteration = 0; iteration < 5; ++iteration) {
-        for (auto &stat : imageData.m_clusters) {
+#pragma omp parallel for private(r, g, b, c)
+        for (int i = 0; i < imageData.m_clusters.size(); ++i) {
+            auto &stat = imageData.m_clusters[i];
             r = 0;
             g = 0;
             b = 0;
@@ -275,7 +331,7 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
             stat.centroid = qRgb(r, g, b);
             stat.ratio = qreal(stat.colors.count()) / qreal(imageData.m_samples.count());
             stat.colors = QList<QRgb>({stat.centroid});
-        }
+        } // END omp parallel for
 
         for (auto color : std::as_const(imageData.m_samples)) {
             positionColor(color, imageData.m_clusters);
