@@ -15,7 +15,11 @@ ShadowedBorderRectangleMaterial::ShadowedBorderRectangleMaterial()
     setFlag(QSGMaterial::Blending, true);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QSGMaterialShader *ShadowedBorderRectangleMaterial::createShader() const
+#else
+QSGMaterialShader *ShadowedBorderRectangleMaterial::createShader(QSGRendererInterface::RenderMode) const
+#endif
 {
     return new ShadowedBorderRectangleShader{shaderType};
 }
@@ -46,6 +50,7 @@ ShadowedBorderRectangleShader::ShadowedBorderRectangleShader(ShadowedRectangleMa
     setShader(shaderType, QStringLiteral("shadowedborderrectangle"));
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void ShadowedBorderRectangleShader::initialize()
 {
     ShadowedRectangleShader::initialize();
@@ -65,3 +70,22 @@ void ShadowedBorderRectangleShader::updateState(const QSGMaterialShader::RenderS
         p->setUniformValue(m_borderColorLocation, material->borderColor);
     }
 }
+#else
+bool ShadowedBorderRectangleShader::updateUniformData(QSGMaterialShader::RenderState &state, QSGMaterial *newMaterial, QSGMaterial *oldMaterial)
+{
+    bool changed = ShadowedRectangleShader::updateUniformData(state, newMaterial, oldMaterial);
+    QByteArray *buf = state.uniformData();
+    Q_ASSERT(buf->size() >= 160);
+
+    if (!oldMaterial || newMaterial->compare(oldMaterial) != 0) {
+        const auto material = static_cast<ShadowedBorderRectangleMaterial *>(newMaterial);
+        memcpy(buf->data() + 136, &material->borderWidth, 8);
+        float c[4];
+        material->borderColor.getRgbF(&c[0], &c[1], &c[2], &c[3]);
+        memcpy(buf->data() + 144, c, 16);
+        changed = true;
+    }
+
+    return changed;
+}
+#endif
