@@ -369,7 +369,7 @@ void ToolBarLayout::Private::performLayout()
         return;
     }
 
-    qreal maxHeight = moreButtonInstance->isVisible() ? moreButtonInstance->height() : 0.0;
+    qreal maxHeight = 0.0;
     qreal maxWidth = 0.0;
 
     // First, calculate the total width and maximum height of all delegates.
@@ -459,6 +459,10 @@ void ToolBarLayout::Private::performLayout()
         moreButtonInstance->setVisible(false);
     }
 
+    if (moreButtonInstance->isVisible() && !q->heightValid()) {
+        maxHeight = std::max(maxHeight, moreButtonInstance->implicitHeight());
+    };
+
     qreal currentX = layoutStart(visibleActionsWidth);
     for (auto entry : std::as_const(sortedDelegates)) {
         if (!entry->isVisible()) {
@@ -517,7 +521,7 @@ QVector<ToolBarLayoutDelegate *> ToolBarLayout::Private::createDelegates()
     for (auto action : std::as_const(actions)) {
         if (delegates.find(action) != delegates.end()) {
             result.append(delegates.at(action).get());
-        } else {
+        } else if (action) {
             auto delegate = std::unique_ptr<ToolBarLayoutDelegate>(createDelegate(action));
             if (delegate) {
                 result.append(delegate.get());
